////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit DAVresources;
interface

type
 TDAVcmd = ( davPROPFIND,
             davPropPatch, //coming soon
             davMKCOL,
             davGET,
             davHEAD,      //coming soon
             davPOST,      //coming soon
             davDELETE,
             davPUT,
             davCOPY,      //coming soon
             davMOVE,      //coming soon
             davLOCK,
             davUNLOCK);

 TEncode = (encUUEncode, encBase64, encMime);

const
 davStrXMLheader = '<?xml version="1.0" encoding="utf-8" ?>'+#13#10;

 davCmdStr : array[TDAVcmd] of string =
           ( 'PROPFIND',
             'PROPPATCH',
             'MKCOL',
             'GET',
             'HEAD',
             'POST',
             'DELETE',
             'PUT',
             'COPY',
             'MOVE',
             'LOCK',
             'UNLOCK');

 davStrHeader         = '%s %s HTTP/1.1'+#13#10+
                        'Cache-Control: no-cache'+#13#10+
                        'Pragma: no-cache'+#13#10+
                        'Accept: */*'+#13#10+
                        'Accept-Encoding: deflate'+#13#10+
                        'User-Agent: IEDWebDAV/1.0 (compatible; MSIE 6.0; Mozilla 1.0 )';

 davStrHost           = 'Host: %s';
 davStrDepth          = 'Depth: %s';
 davStrDest           = 'Destination: %s';
 davStrOverwrite      = 'Overwrite: %s';
 davStrTimeout        = 'Timeout: Infinite, Second-%d';
 davStrAuthor         = 'Authorization: Basic %s';
 davStrPproxyAuthor   = 'Proxy-Authorization: Basic %s';
 davStrLockToken      = 'Lock-Token: <%s>';
 davRequestURI        = 'Request-URI: %s';
 davStrCType          = 'Content-Type: %s'; //text/xml; charset="utf-8"
 davStrCLength        = 'Content-Length: %d';
 davStrStatus         = 'Status-URI : %s';


 davContentAllProperty       = davStrXMLheader+
                             '<D:propfind xmlns:D="DAV:">'+#13#10+
                             '<D:allprop/>'+#13#10+
                             '</D:propfind>';

 davContentLock              = davStrXMLheader+
                             '<D:lockinfo xmlns:D="DAV:">'+#13#10+
                             '<D:lockscope><D:%s/></D:lockscope>'+#13#10+
                             '<D:locktype><D:%s/></D:locktype>'+#13#10+
                             '<D:owner>%s</D:owner></D:lockinfo>';

 davContentCheckActiveLock   = davStrXMLheader+
                             '<D:propfind xmlns:D="DAV:">'+#13#10+
                             '<D:prop><D:lockdiscovery/></D:prop>'+#13#10+
                             '</D:propfind>';

function EncodeString(Encoding: TEncode; SrcData: string) : String;
implementation

Const
    bin2uue  : String = '`!"#$%&''()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_';
    bin2b64  : String = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
    uue2bin  : String = ' !"#$%&''()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_ ';
    b642bin  : String = '~~~~~~~~~~~^~~~_TUVWXYZ[\]~~~|~~~ !"#$%&''()*+,-./0123456789~~~~~~:;<=>?@ABCDEFGHIJKLMNOPQRS';

function EncodeString(Encoding: TEncode; SrcData: string) : String;
var
    Offset : Integer;
    Pos1   : Integer;
    Pos2   : Integer;
    I      : Integer;
    Size   : Integer;
begin
    Size:= Length(SrcData);
    SetLength(Result, Size * 4 div 3 + 4);
    FillChar(Result[1], Size * 4 div 3 + 2, #0);

    if Encoding = encUUEncode then begin
        Result[1] := Char(((Size - 1) and $3f) + $21);
        Size      := ((Size + 2) div 3) * 3;
    end;
    Offset := 2;
    Pos1   := 0;
    Pos2   := 0;
    case Encoding of
        encUUEncode: Pos2 := 2;
          encBase64,
            encMime: Pos2 := 1;
    end;
    Result[Pos2] := #0;

    while Pos1 < Size do begin
        if Offset > 0 then begin
            Result[Pos2] := Char(ord(Result[Pos2]) or
                                 ((ord(SrcData[Pos1]) and
                                  ($3f shl Offset)) shr Offset));
            Offset := Offset - 6;
            Inc(Pos2);
            Result[Pos2] := #0;
        end
        else if Offset < 0 then begin
            Offset := Abs(Offset);
            Result[Pos2] := Char(ord(Result[Pos2]) or
                                 ((ord(SrcData[Pos1]) and
                                  ($3f shr Offset)) shl Offset));
            Offset := 8 - Offset;
            Inc(Pos1);
        end
        else begin
            Result[Pos2] := Char(ord(Result[Pos2]) or
                                 ((ord(SrcData[Pos1]) and $3f)));
            Inc(Pos2);
            Inc(Pos1);
            Result[Pos2] := #0;
            Offset    := 2;
        end;
    end;

    case Encoding of
    encUUEncode:
        begin
            if Offset = 2 then
                Dec(Pos2);
            for i := 2 to Pos2 do
                Result[i] := bin2uue[ord(Result[i])+1];
        end;
    encBase64, encMime:
        begin
            if Offset = 2 then
                Dec(Pos2);
            for i := 1 to Pos2 do
                Result[i] := bin2b64[ord(Result[i])+1];
            while (Pos2 and 3) <> 0  do begin
                Inc(Pos2);
                Result[Pos2] := '=';
            end;
        end;
    end;
    SetLength(Result, Pos2);
end;

function davStatusStr(Status : integer):string;
begin
 Case Status of
   102 : result:='Processing';
   207 : result:='Multi-Stat';
   422 : result:='Unprocessable Entity';
   423 : result:='Locked';
   424 : result:='Failed Dependency';
   507 : result:='Insufficient Storage';
   201 : result:='Created';
   403 : result:='Forbidden';
   405 : result:='Method Not Allowed';
   409 : result:='Conflict';
   415 : result:='Unsupported Media Type';
   412 : result:='Precondition Failed';
   502 : result:='Bad Gateway';
   204 : result:='No Content';
   200 : result:='OK';
   301 : result:='Moved Permanently';
   304 : result:='Not Modified';
   400 : result:='Bad Request';
   401 : result:='Unauthorized';
   404 : result:='Object Not Found';
   500 : result:='Internal Server Error';
   501 : result:='Not Implemented';
  else result:='Unknown';
  end
end;

end.
