////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit UnParserShapes;

interface
uses
  Windows, Controls, Classes, Forms, ExtCtrls, Graphics, Messages,
  IEShapes, HREFParser, UnConfig;

var
   ShapeOffsetX : integer = 50;
   ShapeOffsetY : integer = 50;
   ShapeDeltaX  : integer = 150;
   ShapeDeltaY  : integer = 60;

   //feUnknown,feRoot,feHTML,feIMG,feMEDIA,feJava,feAX,feStyle,feScript,feASP,fePHP,fePERL
   ImageByExt   : array[TExtentions] of integer = (1,11,2,9,4,10,5,7,0,0,3,0);

type

  TIEParserShape = class(TIEBitmapShape)
  private
    FHREFdoc: THREFDocument;
    FURL: string;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    property URL : string read FURL write FURL;
    property HREFdoc : THREFDocument read FHREFdoc write FHREFdoc;
  end;

procedure ClearParent(AParent: TWinControl);
function  ShapeOf(AParent: TWinControl; AURL : string):TIEParserShape;
function  AddShape(AParent: TWinControl;  Img:TImageList;
                   Src: THREF; Title:string):TIEParserShape;
procedure AddLink(AParent: TWinControl; Src,Dest: TIEParserShape;
                  clBroken,clUnchecked,clRemote,clLocal:TColor);
function  GetSelectedShape(AParent : TWinControl): TIEParserShape;

implementation

var
   LevelYPositions : array [0..1024] of integer;

{ TIEParserShape }
constructor TIEParserShape.Create(AOwner: TComponent);
begin
  inherited;
  FHREFdoc:=THREFDocument.Create;
end;

destructor TIEParserShape.Destroy;
begin
  if Assigned(FHREFdoc) then FHREFdoc.Free;
  inherited;
end;

function  ShapeOf(AParent: TWinControl; AURL : string):TIEParserShape;
 var i : integer;
begin
 result := nil;
 if not Assigned(AParent) then exit;
 for i:=0  to AParent.ControlCount-1 do
  if (AParent.Controls[i] is TIEParserShape) then
   begin
     if (AParent.Controls[i] as TIEParserShape).URL = AURL then
      begin
       result:= AParent.Controls[i] as TIEParserShape;
       break;
      end;
   end;
end;

procedure AddLink(AParent: TWinControl; Src,Dest: TIEParserShape;
                  clBroken,clUnchecked,clRemote,clLocal:TColor);
var
  Link : TIEConnector;
  i : integer;
begin
 if (Src=nil)or(Dest=nil) then exit;
 //Check duplicate
 for i:=0 to AParent.ComponentCount-1 do
  if AParent.Components[i] is TIEConnector then
   With (AParent.Components[i] as TIEConnector) do
    if (StartConn.Shape=Src)and(StartConn.Shape=Dest) then
      exit;
 //Create new
 Link := TIEConnector.Create(nil);
 Link.Parent := AParent;
 Link.StartConn.Shape := TControl(Src);
 Link.StartConn.ArrowType := atArrowOut;//atTerminator;
 Link.StartConn.Side := csRight;
 Link.EndConn.Shape := TControl(Dest);
 Link.EndConn.ArrowType := atArrowIn;
 Link.EndConn.Side := csLeft;
 if frmConfig.cbColored.Checked then
  Case Dest.HREFdoc.HREFSelf.State of
   lsUnchecked : Link.LineColor := clUnchecked;
   lsBroken : Link.LineColor := clBroken;
   lsGood : begin
            if Dest.HREFdoc.HREFSelf.ProtocolType in [kpHTTP,kpFTP] then
             Link.LineColor := clRemote //clGray
            else
             Link.LineColor := clLocal;
           end;
  end
 else
  Link.LineColor := clLocal;

 Link.SendToBack;
 Link.SetBoundingRect;
end;

function AddShape(AParent: TWinControl; Img:TImageList;
                  Src: THREF; Title:string):TIEParserShape;
begin
 result := ShapeOf(AParent,Src.UniqueDocURL);
 if not Assigned(result) then
  begin
   result:= TIEParserShape.Create(AParent.Owner);
   with result do
    begin
     Parent   := AParent;
     Top      := ShapeOffsetY+LevelYPositions[Src.Level];
     Left     := ShapeOffsetX+ShapeDeltaX*Src.Level;
     Hint     := Src.UniqueDocURL;
     ShowHint := true;
     URL      := Src.UniqueDocURL;
     Images   := Img;
     inc(LevelYPositions[Src.Level],ShapeDeltaY);
     if Title<>'' then
      begin
       ShapeCaption := TIETextShape.Create(AParent.Owner);
       ShapeCaption.Text := Title;
       AlignCaption;
      end;
     ImageIndex:=ImageByExt[Src.GetFileType];
    end;
   end;
end;

procedure ClearParent(AParent: TWinControl);
begin
 FillChar(LevelYPositions,SizeOf(LevelYPositions),0);
 TIECustomShape.DeleteAllShapes(AParent);
end;

function GetSelectedShape(AParent : TWinControl): TIEParserShape;
var i : integer;
begin
result := nil;
  if Assigned(AParent) then
   for i:=0 to AParent.ControlCount-1 do
      if (AParent.Controls[i] is TIEParserShape) and
        (TIEParserShape(AParent.Controls[i]).Selected) then
         begin
          result:= TIEParserShape(AParent.Controls[i]);
          exit;
         end;
end;


initialization
  RegisterClasses([TIEParserShape]);
end.
