// IEDXMLReader 1.0b                                                          //
////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit IEDXML;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls;

type
  TSearchFlag  = (schPartial,schUpper,schNoSelf);
  TSearchFlags = set of TSearchFlag;

  TIEDXMLNodes = class;
  TIEDXMLNode = class;
  TIEDXMLReader = class;

  TIEDXMLNodes = class(TList)
  private
    function  GetNode(Idx: integer): TIEDXMLNode;
  protected
  public
    procedure  Clear; override;
    destructor Destroy; override;

    property Node[Idx: integer]: TIEDXMLNode read GetNode; default;
  end;

  TIEDXMLNode = class(TObject)
  private
    FName: string;
    FUName: string;
    FParent: TIEDXMLNode;
    FNodes: TIEDXMLNodes;
    FAttributes: string;
    FValue: string;
    FSingle: boolean;
    procedure SetName(const Value: string);
  protected
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   Clear;
    procedure   AddNode(ANode: TIEDXMLNode); virtual;
    procedure   ParseAttributes(Const Attr: string);
    procedure   ParseHeader(Const Header: string);
    function    Text: string;
    function    IsSameName(AName : string): boolean;
    function    NodeOf(SubStr: string;
                       Flags: TSearchFlags;
                       Depth:integer): TIEDXMLNode;


    property IsSingle   : boolean read FSingle;
    property Parent     : TIEDXMLNode read FParent write FParent;
    property Name       : string read FName write SetName;   //Tag name
    property Value      : string read FValue write FValue; //Tag value
    property Attributes : string read FAttributes write FAttributes; //Tag attribute
    property Nodes      : TIEDXMLNodes read FNodes;
  end;

  TIEDXMLReader = class(TPersistent)
  private
    FRoot: TIEDXMLNode;
    FCurr: TIEDXMLNode;
  protected
    procedure DoParseNode(pNode: TIEDXMLNode; AStream: TStream);
  public
    constructor Create;
    destructor Destroy; override;

    procedure Parse(AStream : TStream);
    property  Root : TIEDXMLNode read FRoot;
  end;

implementation


{ TIEDXMLNodes }

procedure TIEDXMLNodes.Clear;
var i:integer;
begin
  for i:=Self.Count-1 downto 0 do
    TIEDXMLNode(Self.Items[i]).free;
  inherited;
end;

destructor TIEDXMLNodes.Destroy;
begin
  Clear;
  inherited;
end;

function TIEDXMLNodes.GetNode(Idx: integer): TIEDXMLNode;
begin
 result:= TIEDXMLNode(self.Items[idx]);
end;

{ TIEDXMLNode }

procedure TIEDXMLNode.AddNode(ANode: TIEDXMLNode);
begin
 Nodes.Add(ANode);
 ANode.Parent := self;
end;

procedure TIEDXMLNode.Clear;
begin
  FNodes.Clear;
  FName:= '';
  FUName:= '';
  FParent:= nil;
end;

constructor TIEDXMLNode.Create;
begin
  FName:= '';
  FUName:= '';
  FParent:= nil;
  FNodes:= TIEDXMLNodes.Create;
  FAttributes:= '';
  FValue:='';
end;

destructor TIEDXMLNode.Destroy;
begin
  Clear;
  if Assigned(FNodes) then FNodes.Free;
  inherited;
end;

function TIEDXMLNode.IsSameName(AName: string): boolean;
begin
 result:= Pos(AName,FUName)<>0;
end;

function TIEDXMLNode.NodeOf(SubStr: string;
                            Flags: TSearchFlags;
                            Depth:integer): TIEDXMLNode;
var i : integer;

 function EqualStr(S1: string): boolean;
  begin
   if schUpper in Flags then
    S1:=UpperCase(S1);
   if schPartial in Flags then
    result:= (Pos(SubStr,S1)<>0)
   else
    result:= (SubStr=S1);
  end;

begin
 result:=nil;
 if Depth<1 then exit;
 if schUpper in Flags then
  SubStr:= UpperCase(SubStr);
 if not (schNoSelf in Flags) then
  begin
   if EqualStr(Name) then
    begin
     result:= self;
     exit;
   end
  end
 else
  Flags := Flags -[schNoSelf];
 //Search in child collection
 for i:=0 to Nodes.Count-1 do
  begin
   Dec(Depth);
   result:= Nodes[i].NodeOf(SubStr,Flags,Depth);
   if result<>nil then
    break;
  end;
end;

procedure TIEDXMLNode.ParseAttributes;
var InQuote, InAttribute  : boolean;
    Cache : string;
    Quote : char;
    sa, qp, qa, i : integer;
begin
 InQuote     := false;
 InAttribute := false;
 Quote       :=#0;
 sa :=1; qa:=0; qp:=0;
 for i:=1 to Length(Attr) do
  case Attr[i] of
   '='      : begin
                InAttribute:= true;
                qa := i;
              end;
   '"','''' : if InQuote then
               begin
                if Quote=Attr[i] then
                 begin
                  cache := trim(copy(Attr,sa,qa-sa+1)+copy(Attr,qp+1,i-qp-1));
                  if cache<>'' then
                   begin
                    if FAttributes<>'' then
                      FAttributes := FAttributes+#13+cache
                    else
                      FAttributes := cache;
                   end;
                  sa := i+1;
                  InAttribute := false;
                  InQuote:= false;
                 end;
               end
              else
               begin
                Quote  :=Attr[i];
                InQuote:= true;
                qp:=i;
               end;
   #13,#10  : begin end;
   '>','<'  : if not InQuote then {error};
   ' '      : if (not InAttribute)and(not InQuote) then
               sa:=i+1;
  end;
end;

procedure TIEDXMLNode.ParseHeader(Const Header: string);
var np : integer;
    l  : integer;
begin
 l:=length(Header);
 FSingle :=(Header[l-1] in ['/','?']);
 np := Pos(' ',Header);
 if (np=0) then
 begin
  if (FSingle) then np:=l-1 else np:=l;
 end;
 Name := copy(Header,2,np-2);
 ParseAttributes(copy(Header,np+1,Length(Header) - np));
end;

procedure TIEDXMLNode.SetName(const Value: string);
begin
 if FName<>Value then
  begin
   FName := Value;
   FUName:= UpperCase(FName);
  end;
end;

function TIEDXMLNode.Text: string;
begin
 result:='['+FName+'] '+
         'Attributes ('+ FAttributes+') '+
         'Value ('+FValue+')';
end;

{ TIEDXMLReader }
constructor TIEDXMLReader.Create;
begin
  inherited;
  FRoot := TIEDXMLNode.Create;
end;

destructor TIEDXMLReader.Destroy;
begin
  FRoot.Clear;
  inherited;
end;

procedure TIEDXMLReader.Parse(AStream: TStream);
begin
 FRoot.Clear;
 FCurr:= FRoot;
 DoParseNode(FRoot,AStream);
end;

type
  TChars = set of char;

procedure TIEDXMLReader.DoParseNode(pNode: TIEDXMLNode; AStream: TStream);
var c :  char;
    nNode : TIEDXMLNode;
    cache : string ;

 function ReadNext: boolean;
 var r : integer;
 begin
  cache:=cache+c;
  repeat r := AStream.Read(c,1); until not((c in [#13,#10])and(r>0));
  result:=(r>0);
 end;

 function WaitTagCh(Chars: TChars): boolean;
 begin
  result:= false;
  while ReadNext do
   if (c in Chars)then
    begin
     result:= true;  break;
    end;
 end;

 function WaitTagChs(wc1,wc2,wc3: Char): boolean;
 var ll : integer;
 begin
  result:= false;
  while WaitTagCh([wc3]) do
  begin
   ll := length(Cache); if ll<2 then continue;
   if (Cache[ll-1]=wc1)and(Cache[ll]=wc2) then
    begin
     result:= true;  break;
    end;
  end;
 end;

begin
 cache:=''; c:=' ';
 if pNode=nil then exit;
 if pNode.IsSingle then pNode:= pNode.Parent;
 if pNode=nil then exit;
 WaitTagCh(['<']);
 pNode.Value := trim(Cache);
 Cache:='';
    if not readNext then exit;
    case c of
    '/': begin // '</' - Close parent node
           if not WaitTagCh(['>']) then exit; //check end;
           pNode:= pNode.Parent;
         end;
    '!': begin // '<!' - Comment or DOCKTYPE
           if not readNext then exit;
           case c of
            '-' : WaitTagChs('-','-','>');
            '[' : WaitTagChs(']',']','>');
            else WaitTagCh(['>']);
           end;
         end;
    else
     begin
      if not WaitTagCh(['>']) then exit;
      nNode := TIEDXMLNode.Create;
      nNode.ParseHeader(trim(Cache+c));
      pNode.AddNode(nNode);
      pNode := nNode;
     end;
    end;
    DoParseNode(pNode,Astream);
end;

end.


